import 'package:cramr_ui_lib/model/cardinality_selection.dart';
import 'package:cramr_ui_lib/model/verb_test_model.dart';
import 'package:cramr_ui_lib/routes/verb_indicative_evaluate_route.dart';
import 'package:cramr_ui_lib/settings/settings_controller.dart';
import 'package:cramr_ui_lib/settings/settings_service.dart';
import 'package:cramr_ui_lib/util/cardinality_filter.dart';
import 'package:flutter/material.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:provider/provider.dart';
import 'package:shared_preferences/shared_preferences.dart';

final List<String> _randomIndicative = [
  'exp1',
  'exp2',
  'exp3',
  'exp4',
  'exp5',
  'exp6',
];

final List<String> _randomPronouns = [
  'pron1',
  'pron2',
  'pron3',
  'pron4',
  'pron5',
  'pron6',
];

const String _randomInfinitive = 'infinitive';

void main() {
  testWidgets('Test the content of a 6/6 score', (WidgetTester tester) async {
    SharedPreferences.setMockInitialValues(<String, Object>{'counter': 1});
    SharedPreferences prefs = await SharedPreferences.getInstance();
    await tester.pumpWidget(ChangeNotifierProvider(
      create: (BuildContext context) {
        var model = VerbTestModel();
        model.initialise(_randomIndicative, _randomPronouns, 'fr',
            _randomInfinitive, 'translation');
        int i = 0;
        for (var element in _randomIndicative) {
          model.add(element, i++);
        }
        return model;
      },
      child: MaterialApp(
        home: VerbIndicativeEvaluateRoute(
            SettingsController(SettingsService(prefs), ThemeMode.dark, prefs)),
      ),
    ));
    Finder titleFinderOne = find.text('Vervoeging ');
    Finder titleFinderTwo = find.text(_randomInfinitive);
    expect(titleFinderOne, findsOneWidget);
    expect(titleFinderTwo, findsOneWidget);
    Finder scoreFinder = find.text('6/6');
    expect(scoreFinder, findsOneWidget);
    for (var element in _randomIndicative) {
      final Finder indicativeFinder = find.text(element);
      expect(indicativeFinder, findsOneWidget);
    }
  });

  testWidgets('Test the content of a 3/3 score (singular)',
      (WidgetTester tester) async {
    SharedPreferences.setMockInitialValues(<String, Object>{'counter': 1});
    SharedPreferences prefs = await SharedPreferences.getInstance();
    var selection = CardinalitySelection(true, false);
    var filteredPronounList =
        CardinalityFilter.filteredList(selection, _randomPronouns);
    var filteredIndicativeList =
        CardinalityFilter.filteredList(selection, _randomIndicative);
    await tester.pumpWidget(ChangeNotifierProvider(
      create: (BuildContext context) {
        var model = VerbTestModel();
        model.initialise(filteredIndicativeList, filteredPronounList, 'fr',
            'infinitive', 'translation');
        int i = 0;
        for (var element in filteredIndicativeList) {
          model.add(element, i++);
        }
        return model;
      },
      child: MaterialApp(
        home: VerbIndicativeEvaluateRoute(
            SettingsController(SettingsService(prefs), ThemeMode.dark, prefs)),
      ),
    ));
    Finder titleFinderOne = find.text('Vervoeging ');
    Finder titleFinderTwo = find.text(_randomInfinitive);
    expect(titleFinderOne, findsOneWidget);
    expect(titleFinderTwo, findsOneWidget);
    Finder scoreFinder = find.text(
        '${filteredIndicativeList.length}/${filteredIndicativeList.length}');
    expect(scoreFinder, findsOneWidget);
    for (var element in filteredIndicativeList) {
      final Finder indicativeFinder = find.text(element);
      expect(indicativeFinder, findsOneWidget);
    }
  });

  testWidgets('Test the content of a 3/3 score (plural)',
      (WidgetTester tester) async {
    SharedPreferences.setMockInitialValues(<String, Object>{'counter': 1});
    SharedPreferences prefs = await SharedPreferences.getInstance();
    var selection = CardinalitySelection(false, true);
    var filteredPronounList =
        CardinalityFilter.filteredList(selection, _randomPronouns);
    var filteredIndicativeList =
        CardinalityFilter.filteredList(selection, _randomIndicative);
    await tester.pumpWidget(ChangeNotifierProvider(
      create: (BuildContext context) {
        var model = VerbTestModel();
        model.initialise(filteredIndicativeList, filteredPronounList, 'fr',
            'infinitive', 'translation');
        int i = 0;
        for (var element in filteredIndicativeList) {
          model.add(element, i++);
        }
        return model;
      },
      child: MaterialApp(
        home: VerbIndicativeEvaluateRoute(
            SettingsController(SettingsService(prefs), ThemeMode.dark, prefs)),
      ),
    ));
    Finder titleFinderOne = find.text('Vervoeging ');
    Finder titleFinderTwo = find.text(_randomInfinitive);
    expect(titleFinderOne, findsOneWidget);
    expect(titleFinderTwo, findsOneWidget);
    Finder scoreFinder = find.text(
        '${filteredIndicativeList.length}/${filteredIndicativeList.length}');
    expect(scoreFinder, findsOneWidget);
    for (var element in filteredIndicativeList) {
      final Finder indicativeFinder = find.text(element);
      expect(indicativeFinder, findsOneWidget);
    }
  });

  testWidgets('Test the content of a 5/6 score', (WidgetTester tester) async {
    SharedPreferences.setMockInitialValues(<String, Object>{'counter': 1});
    SharedPreferences prefs = await SharedPreferences.getInstance();
    await tester.pumpWidget(ChangeNotifierProvider(
      create: (BuildContext context) {
        var model = VerbTestModel();
        model.initialise(_randomIndicative, _randomPronouns, 'fr',
            _randomInfinitive, 'translation');
        int i = 0;
        for (var element in _randomIndicative) {
          model.add(element, i++);
        }
        //tamper with 3rd person singular
        model.add('wrong', 2);
        return model;
      },
      child: MaterialApp(
        home: VerbIndicativeEvaluateRoute(
            SettingsController(SettingsService(prefs), ThemeMode.dark, prefs)),
      ),
    ));
    Finder titleFinderOne = find.text('Vervoeging ');
    Finder titleFinderTwo = find.text(_randomInfinitive);
    expect(titleFinderOne, findsOneWidget);
    expect(titleFinderTwo, findsOneWidget);
    Finder scoreFinder = find.text('5/6');
    expect(scoreFinder, findsOneWidget);
    int i = 0;
    for (var element in _randomIndicative) {
      final Finder indicativeFinder = find.text(element);
      expect(indicativeFinder, i == 2 ? findsNothing : findsOneWidget);
      i++;
    }
  });
}
