import 'dart:collection';
import 'package:cramr_ui_lib/model/verb.dart';
import 'package:cramr_ui_lib/repo/verb_repository.dart';
import 'package:cramr_ui_lib/routes/verb_selection_route.dart';
import 'package:cramr_ui_lib/settings/settings_controller.dart';
import 'package:cramr_ui_lib/settings/settings_service.dart';
import 'package:flutter/material.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:shared_preferences/shared_preferences.dart';

void main() {
  List<String> pronouns = ['', '', '', '', '', ''];
  Verb verb1 = Verb(
      'infinitive1',
      'translation1',
      ['singular1', 'singular2', 'singular3', 'plural1', 'plural2', 'plural3'],
      "fr");
  Verb verb2 = Verb(
      'infinitive2',
      'translation2',
      ['singular1', 'singular2', 'singular3', 'plural1', 'plural2', 'plural3'],
      "fr");
  Verb verb3 = Verb(
      'infinitive3',
      'translation3',
      ['singular1', 'singular2', 'singular3', 'plural1', 'plural2', 'plural3'],
      "fr");
  HashMap<String, Verb> verbs = HashMap();
  verbs[verb1.infinitive] = verb1;
  verbs[verb2.infinitive] = verb2;
  verbs[verb3.infinitive] = verb3;
  VerbRepository.instance().initialise(verbs, pronouns);

  testWidgets('Test whether all verbs from the repository get filled in',
      (WidgetTester tester) async {
    SharedPreferences.setMockInitialValues(<String, Object>{'counter': 1});
    SharedPreferences prefs = await SharedPreferences.getInstance();
    await tester.pumpWidget(VerbSelectionRoute(
        SettingsController(SettingsService(prefs), ThemeMode.dark, prefs)));
    // Go through verbs
    VerbRepository.instance().verbs.forEach((key, value) {
      Finder infinitiveFinder = find.text(key);
      expect(infinitiveFinder, findsOneWidget);
      Finder translationFinder = find.text(value.translation);
      expect(translationFinder, findsOneWidget);
      Finder buttonFinder = find.byKey(Key('vervoeg $key'));
      expect(buttonFinder, findsOneWidget);
      Finder startStudyButtonFinder = find.byKey(Key('startStudy_$key'));
      expect(startStudyButtonFinder, findsOneWidget);
    });
  });

  testWidgets('Test the search function', (WidgetTester tester) async {
    SharedPreferences.setMockInitialValues(<String, Object>{'counter': 1});
    SharedPreferences prefs = await SharedPreferences.getInstance();
    await tester.pumpWidget(MaterialApp(
      home: VerbSelectionRoute(
          SettingsController(SettingsService(prefs), ThemeMode.dark, prefs)),
    ));
    Finder searchButtonFinder = find.byKey(const Key("searchVerbButton"));
    expect(searchButtonFinder, findsOneWidget);
    await tester.tap(searchButtonFinder);
    await tester.pumpAndSettle();
    VerbRepository.instance().verbs.forEach((key, value) {
      Finder suggestionTextFinder = find.byKey(Key('suggestedResult_$key'));
      expect(suggestionTextFinder, findsOneWidget);
    });
    await tester.enterText(find.byType(TextField), 'infinitive2');
    await tester.pumpAndSettle();
    await tester.tap(find.byKey(
        Key('suggestedResult_${VerbRepository.instance().verbs.keys.first}')));
    await tester.pumpAndSettle();
    Finder positiveTestFinder =
        find.text(VerbRepository.instance().verbs.keys.first);
    expect(positiveTestFinder, findsOneWidget);
    Finder negativeTestFinder =
        find.text(VerbRepository.instance().verbs.keys.last);
    expect(negativeTestFinder, findsNothing);
  });
}
